from uuid import UUID

from fastapi import APIRouter, Depends, HTTPException

from test_api.services.auth import get_current_user

from ..di import get_user_service
from ..schemas import UserDTO, UserReadDTO
from ..schemas.user_schema import UserWriteDTO
from ..services import UserService

router = APIRouter(
    prefix="/api/v1",
)


@router.get(
    "/users",
    response_model=list[UserReadDTO],
    tags=["users"],
)
async def get_user_list(
    user_service: UserService = Depends(get_user_service),
    user: UserDTO = Depends(get_current_user),
) -> list[UserReadDTO] | str:
    print(user)
    return await user_service.get_all_users()


@router.post("/users", response_model=UserReadDTO, tags=["user"], status_code=201)
async def add_user(
    data: UserWriteDTO,
    user_service: UserService = Depends(get_user_service),
) -> UserReadDTO:
    res = await user_service.add_one(data)
    if not isinstance(res, UserReadDTO):
        raise HTTPException(status_code=400, detail=res)
    return res


@router.get("/user/{user_uuid}", response_model=UserReadDTO, tags=["user"])
async def get_user(
    user_uuid: UUID,
    user_service: UserService = Depends(get_user_service),
) -> UserReadDTO | dict:
    res = await user_service.get_user(id=user_uuid)
    if not isinstance(res, UserReadDTO):
        raise HTTPException(status_code=400, detail=res)
    return res


@router.patch("/user/{user_uuid}", response_model=UserReadDTO, tags=["user"])
async def patch_user(
    user_uuid: UUID,
    data: UserWriteDTO,
    user_service: UserService = Depends(get_user_service),
    user: UserDTO = Depends(get_current_user),
) -> UserReadDTO | dict:
    res = await user_service.patch_one(id=user_uuid, data=data)
    if not isinstance(res, UserReadDTO):
        raise HTTPException(status_code=400, detail=res)
    return res


@router.delete("/user/{user_uuid}", status_code=200, tags=["user"])
async def delete_user(
    user_uuid: UUID,
    user_service: UserService = Depends(get_user_service),
) -> None:
    await user_service.delete_user(id=user_uuid)
    return None
